pro krGhostSliderEventHandler, event

if event.type eq 2 then begin
  widget_control, event.id, get_uvalue=self
  self->updateKnobPosition,event
endif else if event.release eq 1 then begin
  widget_control, event.id, get_uvalue=self
  self->releaseOldWindow
endif
return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::newValue,value

;value is always kept as double, converted later as necessary
value = double(value[0])
if value lt self.minimum then return ;bad entry
if value gt self.maximum then return ;bad entry
;have to get the position of the slider based upon value
if self.vertical eq 1 then begin
  yValue = (value - self.b)/self.slope
  xValue = 0
endif else begin
  xValue = (value - self.b)/self.slope
  yValue = 0
endelse
;have to move back to zero position first, since translate is
;a relative move
ctm = self.oSliderKnobModel->getCTM()
if self.vertical eq 1 then self.oSliderKnobModel->translate,0,-ctm[3,1],0 else $
   self.oSliderKnobModel->translate,-ctm[3,0],0,0
self.oSliderKnobModel->translate,xValue,yValue,0
self.value = value
self.xCurrent = xValue/self.xScale
self.yCurrent = yValue/self.yScale
;have to change to value since self.value is always double
value = call_function(self.dataType ,self.value)
;change the display format
value = strcompress(string(value,format=self.displayFormat),/remove)
self.textObjId->setProperty,string=strcompress(string(value,format=self.displayFormat),/remove)

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::updateKnobPosition,event

xEvent=event.x
yEvent=event.y
if self.vertical eq 1 then begin
  yTrans = (yEvent - self.yCurrent)*self.yScale
  if yTrans eq 0 then return ;don't generate events.
  self.oSliderKnobModel->translate,0,yTrans,0
endif else begin
 xTrans = (xEvent - self.xCurrent)*self.xScale
 if xTrans eq 0 then return ;don't generate events.
 self.oSliderKnobModel->translate,xTrans,0,0
endelse
ctm = self.oSliderKnobModel->getCTM()
;ctm is the current transformation matrix, c[3,0] is the
;current x position
self.value = (ctm[3,self.vertical]*self.slope) + self.b

;print, (yEvent - self.yCurrent)*self.yScale, ctm[3,self.vertical], self.value
;check for out of range conditions.
if self.value lt self.minimum then begin
  self.value = self.minimum
  self->newValue,self.value ;set position by value
endif
if self.value gt self.maximum then begin
  self.value = self.maximum
  self->newValue,self.value ;set position by value
endif
;set the current position in pixels. overrides anything done in newvalue
self.xCurrent = xEvent
self.yCurrent = yEvent

;have to change to value since self.value is always double
value = call_function(self.dataType ,self.value)
self.textObjId->setProperty,string=strcompress(string(value,format=self.displayFormat),/remove)

pseudoEvent = { id: Event.Handler,  top: Event.Top, HANDLER: 0L, $
                         value:value}
call_procedure,self.event_pro,pseudoEvent

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::releaseOldWindow

widget_control,self.drawId, event_pro=self.oldEventPro, $
                 set_uvalue=*self.oldUval
widget_control,self.drawId, draw_motion_events = self.oldMotionEvents

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::eventHandler, event, oWindow, oView

object = oWindow->select(oView, [event.x, event.y])
if not obj_valid(object[0]) then return
if object[0] ne self.oSliderKnobModel then return
if event.press eq 1 then begin
  self.oldEventPro = widget_info(self.drawId, /event_pro)
  self.oldMotionEvents = widget_info(self.drawId, /draw_motion_events)
  widget_control,self.drawId,get_uvalue=oldUval
  self.oldUval = ptr_new(oldUval)
  widget_control,self.drawId, event_pro='krGhostSliderEventHandler',$
                 /draw_motion_events, set_uvalue=self
  oWindow->getProperty,dim=dim
  oView->getProperty,viewplane=viewplane
  self.xScale = viewPlane[2]/dim[0]
  self.yScale = viewPlane[3]/dim[1]
  self.xCurrent = event.x
  self.yCurrent = event.y
endif

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::getProperty, value=value, $
              _ref_extra=extra

self->IDLgrMODEL::GetProperty, _extra=extra
value = self.value

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::setProperty, value=value, $
                  hide=hide, _extra=extra

self->IDLgrMODEL::setProperty, _extra=extra
if n_elements(value) then self.value=value
if n_elements(hide) ne 0 then self->IDLgrMODEL::setProperty,hide=hide

return
end
;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider::cleanup

self->IDLgrMODEL::Cleanup

if ptr_valid(self.oldUval) then ptr_free, self.oldUval
if obj_valid(self.oSliderKnobModel) then obj_destroy,self.oSliderKnobModel
if obj_valid(self.textObjId) then obj_destroy,self.textObjId
if obj_valid(self.oContainer) then obj_destroy,self.oContainer

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

function krGhostSlider::init,drawId,minimum=minimum, maximum=maximum, value=value, $
              event_pro=event_pro, vertical=vertical, xsize=xsize, $
              ysize=ysize, scroll=scroll, title=title, integer=integer, $
              long=long, double=double, float=float,displayFormat=displayFormat,$
              location=location, color=color, oFont=oFont, zVal=zVal, $
              _extra=extra

on_error,2
errno = 0
catch, errno
if errno ne 0 then begin
  catch,/cancel
  message,!error_state.msg + ' Object will not be created'
  return,0
endif
if n_elements(drawId) eq 0 then begin
  message,'The draw widget id that the slider will be on must be present'
  return,0
endif
if not keyword_set(event_pro) then begin
  message,'An event procedure must be specified'
  return,0
endif

;initialize the super class
IF (self->IDLgrModel::Init(_EXTRA=extra) NE 1) THEN RETURN, 0

if not keyword_set(minimum) then minimum=0
if not keyword_set(maximum) then maximum=100
if not keyword_set(value) then begin ;sometimes value is zero
  if arg_present(value) eq 0 then value = (maximum + minimum)/2 else value=0
endif
if not keyword_set(color) then color=[255,255,255]
if n_elements(displayFormat) gt 0 then self.displayFormat=displayFormat
if not keyword_set(zVal)then zVal=0.0

self.drawId = drawId
self.event_pro = event_pro
self.minimum = minimum
self.maximum = maximum
;do the object stuff first
if keyword_set(vertical) then begin
  buttonImage = bytarr(4,xsize,xsize)
  yrange = (ysize - 1*xsize) ;account for size of  buttons
  self.slope = (maximum - minimum)/float(yrange)
  ;find b using the far left point
  self.b = minimum ;- self.slope*xsize
  ;have to get the position of the slider based upon value
  yValue = (value - self.b)/self.slope
  xValue = 0
  self.vertical=1
endif else begin
  buttonImage = bytarr(4,ysize,ysize)
  xrange = (xsize - 1*ysize) ;account for size of  buttons
  self.slope = (maximum - minimum)/float(xrange)
  ;find b using the far left point
  self.b = minimum ;- self.slope*ysize
  ;have to get the position of the slider based upon value
  xValue = (value - self.b)/self.slope
  yValue = 0
endelse

;convert the value to the correct type
if keyword_set(integer) then value=fix(value)  $
  else if keyword_set(long) then value=long(value)  $
    else if keyword_set(double) then value=double(value) else $
      value=float(value)

self.dataType = dataType(value)
;have to correct for the integer case only
if self.dataType eq 'integer' then self.dataType='fix'

;slider knob first
buttonImage[0,*,*] = color[0]
buttonImage[1,*,*] = color[1]
buttonImage[2,*,*] = color[2]

oSliderKnob = obj_new('IDLgrImage', buttonImage, $
              location=[location,zVal])
self.oSliderKnobModel = obj_new('IDLgrModel',select_target=1)
self.oSliderKnobModel->add,oSliderKnob
self.oSliderKnobModel->translate,xValue,yValue,0
self->add,self.oSliderKnobModel

;borders of slider now
xLeft = location[0]
xRight = xLeft + xsize
yBottom = location[1]
yTop = yBottom + ysize
yBuff = ysize/10.
x = [xLeft,xRight, xRight,xLeft, xLeft]
y = [yBottom, yBottom, yTop, yTop, yBottom]
z = fltarr(5) + zVal
oBorder = obj_new('IDLgrPolyline',x,y,z,color=color)
self->add,oBorder
value = strcompress(string(value,format=self.displayFormat),/remove)
self.textObjId = obj_new('IDLgrText', value, location=[xLeft,yTop+yBuff,zVal], COLOR=color)
self->add,self.textObjId
if n_elements(title) ne 0 then begin
  self.titleObjId  = obj_new('IDLgrText', title, location=[xLeft,yBottom-yBuff,zVal], $
                     COLOR=color,vert=1.0)
  self->add,self.titleObjId
endif

self.oContainer = Obj_New('IDL_Container')
self.oContainer->Add, oBorder
self.oContainer->add,oSliderKnob

return,1
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro krGhostSlider__define

void = { krghostslider, $
         inherits IDLgrModel, $
         drawId : 0L, $ ;draw window widget id
         event_pro : '',$
         oldEventPro : '', $
         oldMotionEvents : 0, $
         oContainer : obj_new(), $
         oldUval : ptr_new(), $
         oSliderKnobModel : obj_new(), $ ;knob model for translating
         textObjId : obj_new(), $ ;text object id
         dataType : '', $ ;datatype (float, double,long or int)
         minimum : 0.0d, $ ;slider min value
         maximum : 0.0d, $ ;slider max value
         value : 0.0d, $ ;slider value
         displayFormat : '', $ ;slider display format
         xCurrent : 0.0, $ ;current slider position in pixels (float for precis)
         yCurrent : 0.0, $ ;current slider position in pixels
         xScale : 0.0, $ ;conversion between device and viewplane units
         yScale : 0.0, $ ; "   """
         slope : 0.0d, $ ;slope of data
         b : 0.0d, $ ;intercept
         vertical : 0, $ ;vertical flag
         titleObjId : obj_new(), $ ;title objectfield
         xsize : 0, $ ;xsize
         ysize : 0, $ ;ysize
         scroll :0.0d } ;scroll interval

return
end